import { ObservableArray, isIOS, LayoutBase } from '@nativescript/core';
import { ElementRef, EventEmitter, ɵisListLikeIterable, Component, ChangeDetectionStrategy, Inject, IterableDiffers, ViewChild, ViewContainerRef, ContentChild, TemplateRef, Output, Input, HostListener, Directive, Host, NgModule, NO_ERRORS_SCHEMA } from '@angular/core';
import { NativeScriptRendererFactory, getSingleViewRecursive, registerElement } from '@nativescript/angular';
import { ListViewLinearLayout, ListViewViewType, NG_VIEW, RadListView, ListViewGridLayout, ListViewStaggeredLayout, ReorderHandle } from 'nativescript-ui-listview';

/// <reference path="references.d.ts" />
class ListItemContext extends ElementRef {
    constructor($implicit, item, index, even, odd, category) {
        super(item);
        this.$implicit = $implicit;
        this.item = item;
        this.index = index;
        this.even = even;
        this.odd = odd;
        this.category = category;
    }
}
class RadListViewComponent {
    constructor(_elementRef, _iterableDiffers, _renderer) {
        this._elementRef = _elementRef;
        this._iterableDiffers = _iterableDiffers;
        this._renderer = _renderer;
        this.setupItemView = new EventEmitter();
        this.changeAnimationsEnabled = true;
        this._listView = _elementRef.nativeElement;
        // We should consider setting this default value in the RadListView constructor.
        this._listView.listViewLayout = new ListViewLinearLayout();
        let component = this;
        this._listView.itemViewDisposer = () => {
            if (this.loader) {
                this.loader.clear();
            }
        };
        this._listView.nativeScriptViewAdded = (parent, child) => {
            // Consider exposing this in nativescript-angular as public API
            this._renderer.viewUtil.insertChild(parent, child);
        };
        this._listView.itemViewLoader = (viewType) => {
            switch (viewType) {
                case ListViewViewType.ItemView:
                    if (component._itemTemplate && this.loader) {
                        let nativeItem = this.loader.createEmbeddedView(component._itemTemplate, new ListItemContext(), 0);
                        let typedView = getItemViewRoot(nativeItem);
                        typedView[NG_VIEW] = nativeItem;
                        return typedView;
                    }
                    break;
                case ListViewViewType.ItemSwipeView:
                    if (component._itemSwipeTemplate && this.loader) {
                        let nativeItem = this.loader.createEmbeddedView(component._itemSwipeTemplate, new ListItemContext(), 0);
                        let typedView = getItemViewRoot(nativeItem);
                        typedView[NG_VIEW] = nativeItem;
                        return typedView;
                    }
                    break;
                case ListViewViewType.LoadOnDemandView:
                    if (component._loadOnDemandTemplate && this.loader) {
                        let viewRef = this.loader.createEmbeddedView(component._loadOnDemandTemplate, new ListItemContext(), 0);
                        this.detectChangesOnChild(viewRef, -1);
                        let nativeView = getItemViewRoot(viewRef);
                        nativeView[NG_VIEW] = viewRef;
                        return nativeView;
                    }
                    break;
                case ListViewViewType.HeaderView:
                    if (component._headerTemplate && this.loader) {
                        let viewRef = this.loader.createEmbeddedView(component._headerTemplate, new ListItemContext(), 0);
                        this.detectChangesOnChild(viewRef, -1);
                        let nativeView = getItemViewRoot(viewRef);
                        nativeView[NG_VIEW] = viewRef;
                        return nativeView;
                    }
                    break;
                case ListViewViewType.GroupView:
                    if (!this._listView.groupingFunction && this._listView._hasGroupingFunctionChanged) {
                        break;
                    }
                    if (component._groupTemplate && this.loader) {
                        let viewRef = this.loader.createEmbeddedView(component._groupTemplate, new ListItemContext(), 0);
                        this.detectChangesOnChild(viewRef, -1);
                        let nativeView = getItemViewRoot(viewRef);
                        nativeView[NG_VIEW] = viewRef;
                        return nativeView;
                    }
                    break;
                case ListViewViewType.FooterView:
                    if (component._footerTemplate && this.loader) {
                        let viewRef = this.loader.createEmbeddedView(component._footerTemplate, new ListItemContext(), 0);
                        this.detectChangesOnChild(viewRef, -1);
                        let nativeView = getItemViewRoot(viewRef);
                        nativeView[NG_VIEW] = viewRef;
                        return nativeView;
                    }
                    break;
            }
        };
    }
    ngAfterViewInit() {
        this.setItemTemplates();
    }
    get nativeElement() {
        return this._listView;
    }
    get listView() {
        return this._listView;
    }
    set loadOnDemandTemplate(value) {
        this._loadOnDemandTemplate = value;
        this._listView.refresh();
    }
    set headerTemplate(value) {
        this._headerTemplate = value;
        if (this._listView.ios) {
            this._listView.updateHeaderFooter();
        }
        else if (this._listView.android) {
            this._listView['_updateHeader']();
        }
    }
    set groupTemplate(value) {
        this._groupTemplate = value;
    }
    set footerTemplate(value) {
        this._footerTemplate = value;
        if (this._listView.ios) {
            this._listView.updateHeaderFooter();
        }
        else if (this._listView.android) {
            this._listView['_updateFooter']();
        }
    }
    set itemTemplate(value) {
        this._itemTemplate = value;
        this._listView.refresh();
    }
    set itemSwipeTemplate(value) {
        this._itemSwipeTemplate = value;
        this._listView.refresh();
    }
    set items(value) {
        this._items = value;
        let needDiffer = true;
        if (value instanceof ObservableArray) {
            needDiffer = false;
        }
        if (!this._differ) {
            if (needDiffer && ɵisListLikeIterable(value)) {
                this._differ = this._iterableDiffers.find(this._items).create(this.trackBy || ((index, item) => item));
            }
            this._listView.items = this._items;
        }
    }
    ngDoCheck() {
        if (this._differ) {
            let changes = this._differ.diff(this._items);
            if (changes) {
                const changeCounters = {
                    added: 0,
                    moved: 0,
                    removed: 0,
                    changed: 0,
                };
                [
                    { fn: changes.forEachAddedItem, key: 'added' },
                    { fn: changes.forEachMovedItem, key: 'moved' },
                    { fn: changes.forEachRemovedItem, key: 'removed' },
                    { fn: changes.forEachIdentityChange, key: 'changed' },
                ].map(({ fn, key }) => {
                    const collector = [];
                    fn.bind(changes)((record) => {
                        if (typeof record.previousIndex === 'number' && typeof record.currentIndex === 'number') {
                            collector.push(`${record.previousIndex} -> ${record.currentIndex}`);
                        }
                        else {
                            collector.push(`${record.previousIndex || record.currentIndex || 0}`);
                        }
                        changeCounters[key]++;
                    });
                    return `${key}(${collector})`;
                });
                let previousItemCount = 0;
                for (let item in this._listView.items) {
                    previousItemCount++;
                }
                let incomingItemCount = 0;
                for (let item in this._items) {
                    incomingItemCount++;
                }
                const totalReplacement = (changeCounters.added === incomingItemCount && changeCounters.removed === previousItemCount)
                    || changeCounters.added > incomingItemCount
                    || changeCounters.removed > previousItemCount;
                if (!totalReplacement && isIOS) {
                    const collectionView = this._listView.nativeViewProtected;
                    if (collectionView) {
                        const _listView = this._listView;
                        const _items = this._items;
                        const processUpdates = (() => {
                            let nativePaths = NSMutableArray.alloc().init();
                            changes.forEachRemovedItem((record) => {
                                nativePaths.addObject(NSIndexPath.indexPathForRowInSection(record.previousIndex, 0));
                            });
                            if (nativePaths.count) {
                                collectionView.deleteItemsAtIndexPaths(nativePaths);
                            }
                            nativePaths = NSMutableArray.alloc().init();
                            changes.forEachAddedItem((item) => {
                                nativePaths.addObject(NSIndexPath.indexPathForRowInSection(item.currentIndex, 0));
                            });
                            if (nativePaths.count) {
                                collectionView.insertItemsAtIndexPaths(nativePaths);
                            }
                            nativePaths = NSMutableArray.alloc().init();
                            changes.forEachMovedItem((item) => {
                                collectionView.moveItemAtIndexPathToIndexPath(NSIndexPath.indexPathForRowInSection(item.previousIndex, 0), NSIndexPath.indexPathForRowInSection(item.currentIndex, 0));
                            });
                            nativePaths = NSMutableArray.alloc().init();
                            changes.forEachIdentityChange((item) => {
                                nativePaths.addObject(NSIndexPath.indexPathForRowInSection(item.currentIndex, 0));
                            });
                            if (nativePaths.count) {
                                collectionView.reloadItemsAtIndexPaths(nativePaths);
                            }
                            _listView.items = _items;
                        });
                        if (this.changeAnimationsEnabled) {
                            collectionView.performBatchUpdatesCompletion(processUpdates, () => { });
                        }
                        else {
                            UIView.performWithoutAnimation(() => {
                                collectionView.performBatchUpdatesCompletion(processUpdates, () => { });
                            });
                        }
                    }
                }
                else {
                    this._listView.items = this._items;
                    this._listView.refresh();
                }
            }
        }
    }
    onItemLoadingInternal(args) {
        let index = args.index;
        let currentItem = args.view.bindingContext;
        let ngView = args.view[NG_VIEW];
        // Getting angular view from original element (in cases when ProxyViewContainer
        // is used NativeScript internally wraps it in a StackLayout)
        if (!ngView && args.view instanceof LayoutBase && args.view.getChildrenCount() > 0) {
            ngView = args.view.getChildAt(0)[NG_VIEW];
        }
        if (ngView) {
            this.setupViewRef(ngView, currentItem, index);
            this.detectChangesOnChild(ngView, index);
        }
    }
    setupViewRef(viewRef, data, index) {
        const context = viewRef.context;
        context.$implicit = data;
        context.item = data;
        context.category = data ? data.category : "";
        context.index = index;
        context.even = (index % 2 === 0);
        context.odd = !context.even;
        this.setupItemView.next({ view: viewRef, data: data, index: index, context: context });
        // this.detectChangesOnChild(viewRef, -1);
    }
    setLayout(layout) {
        this._listView.listViewLayout = layout;
    }
    detectChangesOnChild(viewRef, index) {
        // Manually detect changes in child view ref
        // TODO: Is there a better way of getting viewRef's change detector
        const childChangeDetector = viewRef;
        childChangeDetector.markForCheck();
        childChangeDetector.detectChanges();
    }
    setItemTemplates() {
        // The itemTemplateQuery may be changed after list items are added that contain <template> inside,
        // so cache and use only the original template to avoid errors.
        this.itemTemplate = this.itemTemplateQuery;
        if (this._templateMap) {
            const templates = [];
            this._templateMap.forEach(value => {
                templates.push(value);
            });
            this.listView.itemTemplates = templates;
        }
    }
    registerTemplate(key, template) {
        if (!this._templateMap) {
            this._templateMap = new Map();
        }
        const keyedTemplate = {
            key,
            createView: () => {
                const viewRef = this.loader.createEmbeddedView(template, new ListItemContext(), 0);
                const resultView = getItemViewRoot(viewRef);
                resultView[NG_VIEW] = viewRef;
                return resultView;
            }
        };
        this._templateMap.set(key, keyedTemplate);
    }
}
RadListViewComponent.decorators = [
    { type: Component, args: [{
                selector: "RadListView",
                template: `
        <DetachedContainer>
            <Placeholder #loader></Placeholder>
        </DetachedContainer>`,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] }
];
RadListViewComponent.ctorParameters = () => [
    { type: ElementRef, decorators: [{ type: Inject, args: [ElementRef,] }] },
    { type: IterableDiffers, decorators: [{ type: Inject, args: [IterableDiffers,] }] },
    { type: NativeScriptRendererFactory, decorators: [{ type: Inject, args: [NativeScriptRendererFactory,] }] }
];
RadListViewComponent.propDecorators = {
    loader: [{ type: ViewChild, args: ["loader", { read: ViewContainerRef, static: false },] }],
    itemTemplateQuery: [{ type: ContentChild, args: [TemplateRef, { read: TemplateRef, static: false },] }],
    setupItemView: [{ type: Output }],
    changeAnimationsEnabled: [{ type: Input }],
    trackBy: [{ type: Input }],
    items: [{ type: Input }],
    onItemLoadingInternal: [{ type: HostListener, args: ["itemLoadingInternal", ['$event'],] }]
};
class ListViewLinearLayoutDirective {
    constructor() { }
}
ListViewLinearLayoutDirective.decorators = [
    { type: Directive, args: [{
                selector: "ListViewLinearLayout"
            },] }
];
ListViewLinearLayoutDirective.ctorParameters = () => [];
class ListViewGridLayoutDirective {
    constructor() { }
}
ListViewGridLayoutDirective.decorators = [
    { type: Directive, args: [{
                selector: "ListViewGridLayout"
            },] }
];
ListViewGridLayoutDirective.ctorParameters = () => [];
class ListViewStaggeredLayoutDirective {
    constructor() { }
}
ListViewStaggeredLayoutDirective.decorators = [
    { type: Directive, args: [{
                selector: "ListViewStaggeredLayout"
            },] }
];
ListViewStaggeredLayoutDirective.ctorParameters = () => [];
class ReorderHandleDirective {
    constructor() { }
}
ReorderHandleDirective.decorators = [
    { type: Directive, args: [{
                selector: "ReorderHandle"
            },] }
];
ReorderHandleDirective.ctorParameters = () => [];
class TKListViewHeaderDirective {
    constructor(owner, template) {
        this.owner = owner;
        this.template = template;
    }
    ngOnInit() {
        this.owner.headerTemplate = this.template;
    }
}
TKListViewHeaderDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkListViewHeader]"
            },] }
];
TKListViewHeaderDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: TemplateRef, decorators: [{ type: Inject, args: [TemplateRef,] }] }
];
class TKListViewFooterDirective {
    constructor(owner, template) {
        this.owner = owner;
        this.template = template;
    }
    ngOnInit() {
        this.owner.footerTemplate = this.template;
    }
}
TKListViewFooterDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkListViewFooter]"
            },] }
];
TKListViewFooterDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: TemplateRef, decorators: [{ type: Inject, args: [TemplateRef,] }] }
];
class TKListViewItemSwipeDirective {
    constructor(owner, template) {
        this.owner = owner;
        this.template = template;
    }
    ngOnInit() {
        this.owner.itemSwipeTemplate = this.template;
    }
}
TKListViewItemSwipeDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkListItemSwipeTemplate]"
            },] }
];
TKListViewItemSwipeDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: TemplateRef, decorators: [{ type: Inject, args: [TemplateRef,] }] }
];
class TKListViewItemDirective {
    constructor(owner, template) {
        this.owner = owner;
        this.template = template;
    }
    ngOnInit() {
        this.owner.itemTemplate = this.template;
    }
}
TKListViewItemDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkListItemTemplate]"
            },] }
];
TKListViewItemDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: TemplateRef, decorators: [{ type: Inject, args: [TemplateRef,] }] }
];
class TKGroupTemplateDirective {
    constructor(owner, template) {
        this.owner = owner;
        this.template = template;
    }
    ngOnInit() {
        this.owner.groupTemplate = this.template;
    }
}
TKGroupTemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkGroupTemplate]"
            },] }
];
TKGroupTemplateDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: TemplateRef, decorators: [{ type: Inject, args: [TemplateRef,] }] }
];
class TKTemplateKeyDirective {
    constructor(templateRef, owner) {
        this.templateRef = templateRef;
        this.owner = owner;
    }
    set tkTemplateKey(value) {
        if (this.owner && this.templateRef) {
            this.owner.registerTemplate(value, this.templateRef);
        }
    }
}
TKTemplateKeyDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkTemplateKey]"
            },] }
];
TKTemplateKeyDirective.ctorParameters = () => [
    { type: TemplateRef },
    { type: RadListViewComponent, decorators: [{ type: Host }] }
];
TKTemplateKeyDirective.propDecorators = {
    tkTemplateKey: [{ type: Input }]
};
class TKListViewLoadOnDemandDirective {
    constructor(owner, template) {
        this.owner = owner;
        this.template = template;
    }
    ngOnInit() {
        this.owner.loadOnDemandTemplate = this.template;
    }
}
TKListViewLoadOnDemandDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkListLoadOnDemandTemplate]"
            },] }
];
TKListViewLoadOnDemandDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: TemplateRef, decorators: [{ type: Inject, args: [TemplateRef,] }] }
];
class TKListViewLayoutDirective {
    constructor(owner, _elementRef) {
        this.owner = owner;
        this._elementRef = _elementRef;
    }
    ngOnInit() {
        let layout = this._elementRef.nativeElement;
        this.owner.setLayout(layout);
    }
}
TKListViewLayoutDirective.decorators = [
    { type: Directive, args: [{
                selector: "[tkListViewLayout]"
            },] }
];
TKListViewLayoutDirective.ctorParameters = () => [
    { type: RadListViewComponent, decorators: [{ type: Inject, args: [RadListViewComponent,] }] },
    { type: ElementRef, decorators: [{ type: Inject, args: [ElementRef,] }] }
];
function getItemViewRoot(viewRef, rootLocator = getSingleViewRecursive) {
    return rootLocator(viewRef.rootNodes, 0);
}
const LISTVIEW_DIRECTIVES = [RadListViewComponent, TKListViewItemDirective, TKListViewItemSwipeDirective, TKListViewHeaderDirective, TKListViewFooterDirective, TKListViewLoadOnDemandDirective, TKListViewLayoutDirective, ListViewGridLayoutDirective, ListViewStaggeredLayoutDirective, ReorderHandleDirective, ListViewLinearLayoutDirective, TKTemplateKeyDirective, TKGroupTemplateDirective];
registerElement("RadListView", () => RadListView);
registerElement("ListViewLinearLayout", () => ListViewLinearLayout);
registerElement("ListViewGridLayout", () => ListViewGridLayout);
registerElement("ListViewStaggeredLayout", () => ListViewStaggeredLayout);
registerElement("ReorderHandle", () => ReorderHandle);
class NativeScriptUIListViewModule {
}
NativeScriptUIListViewModule.decorators = [
    { type: NgModule, args: [{
                declarations: [LISTVIEW_DIRECTIVES],
                exports: [LISTVIEW_DIRECTIVES],
                schemas: [
                    NO_ERRORS_SCHEMA
                ]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { LISTVIEW_DIRECTIVES, ListItemContext, ListViewGridLayoutDirective, ListViewLinearLayoutDirective, ListViewStaggeredLayoutDirective, NativeScriptUIListViewModule, RadListViewComponent, ReorderHandleDirective, TKGroupTemplateDirective, TKListViewFooterDirective, TKListViewHeaderDirective, TKListViewItemDirective, TKListViewItemSwipeDirective, TKListViewLayoutDirective, TKListViewLoadOnDemandDirective, TKTemplateKeyDirective, getItemViewRoot };
//# sourceMappingURL=nativescript-ui-listview-angular.js.map
